window.Gleam.initSearch([{"doc":"modem","title":"modem","content":"# modem\n\n[![Package Version](https://img.shields.io/hexpm/v/modem)](https://hex.pm/packages/modem)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/modem/)\n\n&gt; **modem**: a device that converts signals produced by one type of device (such\n&gt; as a computer) to a form compatible with another (such as a telephone) – [Merriam-Webster](https://www.merriam-webster.com/dictionary/modem)\n\nModem is a little library for Lustre that helps you manage navigation and URLs in\nthe browser. It converts url requests into messages that you can handle in your\napp&#39;s update function. Modem isn&#39;t a router, but it can help you build one!\n\n## Quickstart\n\nGetting started with modem is easy! Most application&#39;s can get by with pattern\nmatching on a url&#39;s path: no complicated router setup required. Let&#39;s see what\nthat looks like with modem:\n\n```sh\ngleam add lustre modem\n```\n\n```gleam\nimport gleam/uri.{type Uri}\nimport lustre\nimport lustre/attribute\nimport lustre/element.{type Element}\nimport lustre/element/html\nimport lustre/effect.{type Effect}\nimport modem\n\npub fn main() {\n  lustre.application(init, update, view)\n}\n\npub type Route {\n  Wibble\n  Wobble\n}\n\nfn init(_) -&gt; #(Route, Effect(Msg)) {\n  #(Wibble, modem.init(on_url_change))\n}\n\nfn on_url_change(uri: Uri) -&gt; Msg {\n  case uri.path_segments(uri.path) {\n    [&quot;wibble&quot;] -&gt; OnRouteChange(Wibble)\n    [&quot;wobble&quot;] -&gt; OnRouteChange(Wobble)\n    _ -&gt; OnRouteChange(Wibble)\n  }\n}\n\npub type Msg {\n  OnRouteChange(Route)\n}\n\nfn update(_, msg: Msg) -&gt; #(Route, Effect(Msg)) {\n  case msg {\n    OnRouteChange(route) -&gt; #(route, effect.none())\n  }\n}\n\nfn view(route: Route) -&gt; Element(Msg) {\n  html.div([], [\n    html.nav([], [\n      html.a([attribute.href(&quot;/wibble&quot;)], [element.text(&quot;Go to wibble&quot;)]),\n      html.a([attribute.href(&quot;/wobble&quot;)], [element.text(&quot;Go to wobble&quot;)]),\n    ]),\n    case route {\n      Wibble -&gt; html.h1([], [element.text(&quot;You&#39;re on wibble&quot;)])\n      Wobble -&gt; html.h1([], [element.text(&quot;You&#39;re on wobble&quot;)])\n    },\n  ])\n}\n```\n\nHere&#39;s a breakdown of what&#39;s happening:\n\n- We define a `Route` type that represents the page or route we&#39;re currently on.\n\n- `modem.init` is an [`Effect`](https://hexdocs.pm/lustre/4.0.0-rc.2/lustre/effect.html#Effect)\n  that intercepts clicks to local links and browser back/forward navigation and\n  lets you handle them.\n\n- `on_url_change` is a function we write that takes an incoming [`Uri`](https://hexdocs.pm/gleam_stdlib/gleam/uri.html#Uri)\n  and converts it to our app&#39;s `Msg` type.\n\n- In our `view` we can just use normal `html.a.` elements: no special link component\n  necessary. Pattern matching on the `Route` type lets us render different content\n  for each page.\n","url":"index.html"},{"doc":"modem","title":"Options","content":"pub type Options {\n  Options(\n    handle_internal_links: Bool,\n    handle_external_links: Bool,\n  )\n}\n\nOptions(handle_internal_links: Bool, handle_external_links: Bool)\n\nhandle_internal_links\n&lt;p&gt;Enable this option if you’d like to trigger your url change handler when\na link to the same domain is clicked. When enabled, internal links will\n&lt;em&gt;always&lt;/em&gt; update the url shown in the browser’s address bar but they &lt;em&gt;won’t&lt;/em&gt;\ntrigger a page load.&lt;/p&gt;\n\nhandle_external_links\n&lt;p&gt;Enable this option if you’d like to trigger your url change handler even\nwhen the link is to some external domain. You might want to do this for\nexample to save some state in your app before the user navigates away.&lt;/p&gt;\n&lt;p&gt;You will need to manually call &lt;a href=&quot;#load&quot;&gt;&lt;code&gt;load&lt;/code&gt;&lt;/a&gt; to actually navigate to\nthe external link!&lt;/p&gt;\n\nSynonyms:\nmodem.Options\nmodem Options","url":"modem.html#Options"},{"doc":"modem","title":"advanced","content":"pub fn advanced(\n  options: Options,\n  handler: fn(Uri) -&gt; a,\n) -&gt; Effect(a)\n Initialise an advanced modem that lets you configure what types of links to\n intercept. Take a look at the [`Options`](#options) type for info on what\n can be configured.\n\nSynonyms:\nmodem.advanced\nmodem advanced","url":"modem.html#advanced"},{"doc":"modem","title":"back","content":"pub fn back(steps: Int) -&gt; Effect(a)\n The browser maintains a history stack of all the url&#39;s the user has visited.\n This function lets you move back the given number of steps in that stack.\n If you reach the beginning of the stack, further attempts to go back will do\n nothing (unfortunately time travel is not quite possible yet).\n\n **Note**: if you navigate back and then [`push`](#push) a new url, you will\n clear the forward history of the stack.\n\n **Note**: you can go _too far back_ and end up navigating the user off your\n app if you&#39;re not careful.\nSynonyms:\nmodem.back\nmodem back","url":"modem.html#back"},{"doc":"modem","title":"forward","content":"pub fn forward(steps: Int) -&gt; Effect(a)\n The browser maintains a history stack of all the url&#39;s the user has visited.\n This function lets you move forward the given number of steps in that stack.\n If you reach the end of the stack, further attempts to go forward will do\n nothing (unfortunately time travel is not quite possible yet).\n\n **Note**: you can go _too far forward_ and end up navigating the user off your\n app if you&#39;re not careful.\n\nSynonyms:\nmodem.forward\nmodem forward","url":"modem.html#forward"},{"doc":"modem","title":"init","content":"pub fn init(handler: fn(Uri) -&gt; a) -&gt; Effect(a)\n Initialise a simple modem that intercepts internal links and sends them to\n your update function through the provided handler.\n\nSynonyms:\nmodem.init\nmodem init","url":"modem.html#init"},{"doc":"modem","title":"load","content":"pub fn load(uri: Uri) -&gt; Effect(a)\n Load a new uri. This will always trigger a full page reload even if the uri\n is relative or the same as the current page.\n\n **Note**: if you load a new uri while the user has navigated using the back\n or forward buttons, you will clear any forward history in the stack!\n\nSynonyms:\nmodem.load\nmodem load","url":"modem.html#load"},{"doc":"modem","title":"push","content":"pub fn push(uri: Uri) -&gt; Effect(a)\n Push a new uri to the browser&#39;s history stack. If the uri has the same domain\n and port as the current page, or the uri is relative, this will not trigger\n a full page reload.\n\n **Note**: if you push a new uri while the user has navigated using the back\n or forward buttons, you will clear any forward history in the stack!\n\nSynonyms:\nmodem.push\nmodem push","url":"modem.html#push"},{"doc":"modem","title":"replace","content":"pub fn replace(uri: Uri) -&gt; Effect(a)\n Replace the current uri in the browser&#39;s history stack. If the uri has the\n same domain and port as the current page, or the uri is relative, this will\n not trigger a full page reload.\n\nSynonyms:\nmodem.replace\nmodem replace","url":"modem.html#replace"},{"doc":"modem","title":"modem","content":" &gt; **modem**: a device that converts signals produced by one type of device\n &gt; (such as a computer) to a form compatible with another (such as a\n &gt; telephone) –  [Merriam-Webster](https://www.merriam-webster.com/dictionary/modem)\n\n Modem is a little library for Lustre that helps you manage navigation and URLs\n in the browser. It converts url requests into messages that you can handle\n in your app&#39;s update function. Modem isn&#39;t a router, but it can help you\n build one!\n","url":"modem.html"}]);